#!/bin/sh

###############################################################################
#
# Autor: Rodrigo Luis Silva
# email: rsilva.14@gmail.com
# site: http://www.dotsharp.com.br
#
# Versao: 1.0
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. A copy of the GNU General Public
# License can be found at LICENSE.
#
###############################################################################

###############################################################################
# Altere as configuracoes no arquivo /etc/sysconfig/yum-repo-sync
###############################################################################

if [ -f /etc/sysconfig/yum-repo-sync ]; then
	. /etc/sysconfig/yum-repo-sync
else
	echo "Falta o arquivo de configuracao /etc/sysconfig/yum-repo-sync"
fi

which --skip-alias createrepo 1>/dev/null 2>/dev/null
if [ $? == 1 ]; then
	echo -e "\nERRO\n"
	echo "Nao encontrei o programa 'createrepo'"
	echo "Favor intalar o pacote 'createrepo' atraves do CD"
	echo -e "\n"
	exit
fi

which --skip-alias rsync 1>/dev/null 2>/dev/null
if [ $? == 1 ]; then
	echo -e "\nERRO\n"
	echo "Nao encontrei o programa 'rsync'"
	echo "Favor intalar o pacote 'rsync' atraves do CD"
	echo -e "\n"
	exit
fi

which --skip-alias diff 1>/dev/null 2>/dev/null
if [ $? == 1 ]; then
	echo -e "\nERRO\n"
	echo "Nao encontrei o programa 'diff'"
	echo "Favor intalar o pacote 'diffutils' atraves do CD"
	echo -e "\n"
	exit
fi

which --skip-alias mktemp 1>/dev/null 2>/dev/null
if [ $? == 1 ]; then
	echo -e "\nERRO\n"
	echo "Nao encontrei o programa 'mktemp'"
	echo "Favor intalar o pacote 'mktemp' atraves do CD"
	echo -e "\n"
	exit
fi

confnum=0

while [ "x$(eval echo '$'NAME_$confnum)x" != "xx" ]; do
	NAME=$(eval echo '$'NAME_$confnum)
	ENABLE=$(eval echo '$'ENABLE_$confnum)
	VERBOSE=$(eval echo '$'VERBOSE_$confnum | tr 'A-Z' 'a-z')
	UPDATE_DIR=$(eval echo '$'UPDATE_DIR_$confnum)
	UPDATE_SITE=$(eval echo '$'UPDATE_SITE_$confnum)
	EXCLUDE=$(eval echo '$'EXCLUDE_$confnum)
	COMPRESS=$(eval echo '$'COMPRESS_$confnum)
	DELETE=$(eval echo '$'DELETE_$confnum)
	RSYNC_OP=$(eval echo '$'RSYNC_OP_$confnum)

	case "${ENABLE}" in
		yes|true|1)	ENABLE="true";;
		no|false|2)	ENABLE="false";;
	esac

	if [ $ENABLE == "false" ]; then
		confnum=$(($confnum+1))
		continue
	fi

	case "${VERBOSE}" in
		yes|true|1)	VERBOSE="true";;
		no|false|2)	VERBOSE="false";;
	esac

	if [ ${VERBOSE} == "true" ]; then
		echo "Criando diretorios temporarios"
	fi
	ANTES=`/bin/mktemp /tmp/diff.XXXXX`
	DEPOIS=`/bin/mktemp /tmp/diff.XXXXX`
	TMPDIR=`/bin/mktemp -d /tmp/repodata.XXXXX`
	EXCLUDE_FILE=`/bin/mktemp /tmp/exclude.XXXXX`

	if [ $VERBOSE == "true" ]; then
		RSYNC_VERBOSE_OP='--verbose --progress'
		REPO_VERBOSE_OP=''

		echo -e "\n\n"
		echo "#################### $NAME ####################"
		echo -e "\n\n"

	else
		RSYNC_VERBOSE_OP='--quiet'
		REPO_VERBOSE_OP='-q'
	fi

	if [ "x${EXCLUDE}" != "x" ]; then
		echo ${EXCLUDE} | sed 's/,/\n/g' | while read line
		do
			echo ${line} >> ${EXCLUDE_FILE}
		done
	fi

	if [ "x${COMPRESS}" != "x" ]; then
		RSYNC_COMPRESS_OP="--compress"
	fi

	if [ "x${DELETE}" != "x" ]; then
		RSYNC_DELETE_OP="--delete --delete-excluded"
	fi

	if [ ! -d ${UPDATE_DIR} ]; then
		if [ ${VERBOSE} == "true" ]; then
			echo "Criando o diretorio de destino"
		fi
		mkdir -p ${UPDATE_DIR}
	fi

	if [ -d ${UPDATE_DIR}/repodata ]; then
		if [ ${VERBOSE} == "true" ]; then
			echo "Movendo diretorio repodata"
		fi
		/bin/mv ${UPDATE_DIR}/repodata/ ${TMPDIR}
	else
		mkdir ${TMPDIR}/repodata/
	fi

	if [ ${VERBOSE} == "true" ]; then
		echo "Gerando lista de arquivos"
	fi
	/bin/ls -1 ${UPDATE_DIR}/ | sort > ${ANTES}


	if [ ${VERBOSE} == "true" ]; then
		echo -e "-n"
		echo "Iniciando o sincronismo dos arquivos"
		echo -e "-n"
		echo "rsync ${RSYNC_OP} ${RSYNC_VERBOSE_OP} --exclude-from=${EXCLUDE_FILE} ${RSYNC_COMPRESS_OP} ${RSYNC_DELETE_OP} --archive ${UPDATE_SITE} ${UPDATE_DIR}"
	fi
	rsync ${RSYNC_OP} ${RSYNC_VERBOSE_OP} --exclude-from=${EXCLUDE_FILE} ${RSYNC_COMPRESS_OP} ${RSYNC_DELETE_OP} --archive ${UPDATE_SITE} ${UPDATE_DIR}

	if [ ${VERBOSE} == "true" ]; then
		echo "Gerando lista de arquivos"
	fi
	/bin/ls -1 ${UPDATE_DIR} | sort > ${DEPOIS}

	if [ ${VERBOSE} == "true" ]; then
		echo "Checando se foram recebidos novos arquivos"
	fi
	/usr/bin/diff ${ANTES} ${DEPOIS} > /dev/null
	if [ $? == 1 ]; then
		if [ ${VERBOSE} == "true" ]; then
			echo "Gerando indice de todos os arquivos"
		fi
		/usr/bin/createrepo ${REPO_VERBOSE_OP} ${UPDATE_DIR}
	else
		if [ ${VERBOSE} == "true" ]; then
			echo "Movendo diretorio repodata"
		fi
		/bin/mv ${TMPDIR}/repodata/ ${UPDATE_DIR}
	fi

	if [ ${VERBOSE} == "true" ]; then
		echo "Removendo arquivos temporarios"
	fi
	/bin/rm -rf ${ANTES} ${DEPOIS} ${TMPDIR} ${EXCLUDE_FILE}

	confnum=$(($confnum+1))
done
